<?php

/*

    do-record.sh の中からwgetで呼び出されて recfriio の動作状況を保存する

*/

require_once( 'include.php' );

$errmsg = '';

$lock = new Lock( RECLOCKDIR );

$id = 0;
if( isset( $_GET[ 'id' ] ) ) $id = $_GET[ 'id' ];

$pid = 0;
if( isset( $_GET[ 'pid' ] ) ) $pid = $_GET[ 'pid' ];

$devnull = 0;
if( isset( $_GET[ 'devnull' ] ) ) $devnull = $_GET[ 'devnull' ];

$operate = OP_NONE;
if( isset( $_GET[ 'prep' ] ) ) $operate = OP_PREP;
else if( isset( $_GET[ 'rec' ] ) ) $operate = OP_REC;
else if( isset( $_GET[ 'stop' ] ) ) $operate = OP_STOP;

$ch = '';
$name = '';
$reservfile = get_vardir().RESERVFILE;
$is_reserv = FALSE; // 予約録画か

try{

    $statusfile = get_statusfile();
    $status_array = array();
    if( file_exists( $statusfile ) ) read_csv( $statusfile, $status_array );

    $settingfile = get_settingfile();
    $setting_array = array();
    if( file_exists( $settingfile  ) ) read_csv( $settingfile, $setting_array );

    $reservfile = get_reservfile();
    $reserv_array = array();
    if( file_exists( $reservfile ) ) read_csv( $reservfile, $reserv_array );


    $setting = get_setting_by_id( $id, $setting_array );
    if( ! $setting ){

         $is_reserv = TRUE;
         $setting = get_setting_by_id( $id, $reserv_array );
    }
    if( ! $setting ) throw new Exception( 'ID='.$id.'の録画設定が見つかりません' );

    $ch = $setting[ SET_CHANNEL ];
    $name = $setting[ SET_NAMEBASE ];

    // 予約録画準備
    if( $operate == OP_PREP ){

        // チューナ数が足り無い場合は重要性の低いスレッドから停止していく
        $threads = count( $status_array );
        if( $threads >= TUNERNUM ) do{

            // 録画保存していないスレッド
            foreach( $status_array as $status ){
 
                if( $status[ NAMEBASE ] == '-' ){

                   stop_recfriio( $status[ PID ] );
                   $errmsg = $errmsg.' stop(ch = '.$status[ CHANNEL].' name = '.$status[ NAMEBASE ].')';
                   
                    --$threads;
                   if( $threads < TUNERNUM ) break;
                }
            }
            if( $threads < TUNERNUM ) break;

            // 2分以内に停止するスレッド
            foreach( $status_array as $status ){
 
                if( $status[ STARTTIME ] + $status[ DURATION ] <= time() + 120 ){

                   stop_recfriio( $status[ PID ] );
                   $errmsg = $errmsg.' stop(ch = '.$status[ CHANNEL].' name = '.$status[ NAMEBASE ].')'; 
                   
                    --$threads;
                   if( $threads < TUNERNUM ) break;
                }
            }
            if( $threads < TUNERNUM ) break;

            // 予約録画では無いスレッド
            foreach( $status_array as $status ){
 
                if( ! $status[ RESERV ] ){

                   stop_recfriio( $status[ PID ] );
                   $errmsg = $errmsg.' stop(ch = '.$status[ CHANNEL].' name = '.$status[ NAMEBASE ].')'; 
                    
                    --$threads;
                   if( $threads < TUNERNUM ) break;
                }
            }
            if( $threads < TUNERNUM ) break;

        } while( 0 );
    }

    // 録画開始
    if( $operate == OP_REC ){

        $status = get_default_status();
        $status[ ID ] = $id;
        $status[ PID ] = $pid;
        $status[ CHANNEL ] = $setting[ SET_CHANNEL ];
        $status[ STARTTIME ] = time();
        $status[ DURATION ] = $setting[ SET_DURATION ];
        if( $setting[ SET_SLEEPSEC ] ) $status[ DURATION ] += ( 60 - $setting[ SET_SLEEPSEC ] );
        $status[ OUTDIR ] = $setting[ SET_OUTDIR ];

        if( ! $devnull ) $status[ NAMEBASE ] = $setting[ SET_NAMEBASE ];
        else $status[ NAMEBASE ] = '-';

        $status[ UDP ] = $setting[ SET_UDP ];
        $status[ IP ] = $setting[ SET_IP ];
        $status[ PORT ] = $setting[ SET_PORT ];
        $status[ RESERV ] = $is_reserv;

        $status_array[] = $status;
        save_csv( $statusfile, $status_array );
    }

    // 録画完了
    if( $operate == OP_STOP ){

        remove_status( $status_array );

        // 予約情報更新
        if( $is_reserv ){

            if( $setting[ SET_MODE ] == MODE_ONCE ){

                remove_reserv( $reserv_array, $id );

                $reservfile = get_reservfile();
                if( count( $reserv_array ) ) save_csv( $reservfile, $reserv_array );
                else unlink( $reservfile );
            }

            // 次の録画時間まで近い(5分以内)の時は以後の処理を終了
            if( ! count( $reserv_array ) || get_nexttime_array( $reserv_array ) > time() + 5 * 60 ){

                regist_crontab( $reserv_array );
            }
        }
    }
}
catch( Exception $e ){ $errmsg = $e->getMessage(); }


// ログ保存
// エラー時か予約録画の時だけログを残す
if( $errmsg || ( $is_reserv && $operate != OP_NONE ) ){

    $log = 'id='.$id.', ch='.$ch.', name='.$name.' , '.$operate;
    if( $errmsg ) $log = $log.' , '.$errmsg;

    write_log( $log );
}

print( 'ok' );

?>
